// -*-C++-*- cxxrt/auto_ptr.h
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 1998 Dietmar Kuehl, Claas Solutions GmbH > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Claas Solutions make no > 
// <!! representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar.kuehl@claas-solutions.de 
// Title:  auto_ptr - use with care!

// -------------------------------------------------------------------------- 

#if !defined(_CXXRT_AUTO_PTR_H__)
#define _CXXRT_AUTO_PTR_H__ 1

#if !defined(_CXXRTCF_H__)
#  include <cxxrtcf.h>
#endif

_CXXRT_NAMESPACE_BEGIN

// --------------------------------------------------------------------------

template <class _CS_T>
class auto_ptr
{
private:
	template <class _CS_S>
	struct auto_ptr_ref
	{
		mutable auto_ptr<_CS_S>& _CS_m_aptr;
		auto_ptr_ref(auto_ptr<_CS_S>& _CS_ap): _CS_m_aptr(_CS_ap) {}
		auto_ptr_ref(auto_ptr_ref const& _CS_ap): _CS_m_aptr(_CS_ap._CS_m_aptr) {}
	};

public:
	explicit auto_ptr(_CS_T* _CS_p = 0) throw(): _CS_m_ptr(_CS_p) {}
	auto_ptr(auto_ptr& _CS_ap) throw(): _CS_m_ptr(_CS_ap.release()) {}
	template <class _CS_S>
	auto_ptr(auto_ptr<_CS_S>& _CS_ap) throw(): _CS_m_ptr(_CS_ap.release()) {}

	auto_ptr(auto_ptr_ref<_CS_T> _CS_apr) throw():
		_CS_m_ptr(_CS_apr._CS_m_aptr.release())
	{}

	auto_ptr& operator= (auto_ptr& _CS_ap) throw()
		{
			if (&_CS_ap != this)
			  reset(_CS_ap.release());
			return *this;
		}
	template <class _CS_S>
	auto_ptr& operator= (auto_ptr<_CS_S>& _CS_ap) throw()
		{
			if (&_CS_ap != this)
				reset(_CS_ap.release());
			return *this;
		}

	~auto_ptr() throw() { delete _CS_m_ptr; }

	_CS_T& operator* () const throw() { return *_CS_m_ptr; }
	_CS_T* operator-> () const throw() { return _CS_m_ptr; }
	_CS_T* get() const throw() { return _CS_m_ptr; }
	_CS_T* release() throw()
		{
			_CS_T* _CS_rc = _CS_m_ptr;
			_CS_m_ptr = 0;
			return _CS_rc;
		}
	void reset(_CS_T* p) throw() { delete _CS_m_ptr; _CS_m_ptr = p; }

	template <class _CS_S>
	operator auto_ptr_ref<_CS_S> () throw() { return auto_ptr_ref<_CS_S>(*this); }

private:
	_CS_T* _CS_m_ptr;
};

// --------------------------------------------------------------------------

_CXXRT_NAMESPACE_END

#endif /* _CXXRT_AUTO_PTR_H__ */
