// -*-C++-*- cxxrt/collate.h
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 1998 Dietmar Kuehl, Claas Solutions GmbH > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Claas Solutions make no > 
// <!! representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar.kuehl@claas-solutions.de 
// Title:  Declarations for the collate facet

// -------------------------------------------------------------------------- 

#if !defined(_CXXRT_COLLATE_H__)
#define _CXXRT_COLLATE_H__ 1

#if !defined(_CXXRT_LOCALE_H__)
#  include <cxxrt/locale.h>
#endif 
#if !defined(_CXXRT_BSTR_FWD_H__)
#  include <cxxrt/bstr_fwd.h>
#endif

// -------------------------------------------------------------------------

_CXXRT_NAMESPACE_BEGIN

// --------------------------------------------------------------------------

template <class _CS_cT>
class collate: public locale::facet
{
public:
  static locale::id id;

  typedef _CS_cT               char_type;
  typedef basic_string<_CS_cT> string_type;

  _CXXRT_EXPLICIT collate(size_t = 0);
  
  int         compare(_CS_cT const*, _CS_cT const*,
                      _CS_cT const*, _CS_cT const*) const;
  string_type transform(_CS_cT const*, _CS_cT const*) const;
  long        hash(_CS_cT const*, _CS_cT const*) const;

protected:
  virtual ~collate();

  virtual int         do_compare(_CS_cT const*, _CS_cT const*,
                                 _CS_cT const*, _CS_cT const*) const;
  virtual string_type do_transform(_CS_cT const*, _CS_cT const*) const;
  virtual long        do_hash(_CS_cT const*, _CS_cT const*) const;
};

// --------------------------------------------------------------------------

template <class _CS_cT> class collate_byname: public collate<_CS_cT>
{
public:
  typedef _CXXRT_TYPENAME collate<_CS_cT>::string_type string_type;

  collate_byname(char const*, size_t = 0);

protected:
  virtual int         do_compare(_CS_cT const*, _CS_cT const*,
                                 _CS_cT const*, _CS_cT const*) const;
  virtual string_type do_transform(_CS_cT const*, _CS_cT const*) const;
  virtual long        do_hash(_CS_cT const*, _CS_cT const*) const;
};

// --------------------------------------------------------------------------

template <>
inline bool has_facet<collate<char> >(locale const&) 
{
  return true;
}

template <>
inline collate<char> const& use_facet<collate<char> >(locale const& _CS_l)
{
  return _CS_use_facet<collate<char> >(_CS_l);
}

template <>
inline bool has_facet<collate<wchar_t> >(locale const&) 
{
  return true;
}
template <>
inline collate<wchar_t> const& use_facet<collate<wchar_t> >(locale const& _CS_l)
{
  return _CS_use_facet<collate<wchar_t> >(_CS_l);
}

// --------------------------------------------------------------------------

template <class _CS_cT, class _CS_Tr, class _CS_Al>
inline
bool locale::operator() (basic_string<_CS_cT, _CS_Tr, _CS_Al> const &_CS_s1,
                         basic_string<_CS_cT, _CS_Tr, _CS_Al> const &_CS_s2) const
{
  typedef _CXXRT_STD_NAME ::collate<_CS_cT> _CS_collate;
  _CS_collate const &_CS_coll = use_facet<_CS_collate>(*this);
  return _CS_coll.compare(_CS_s1.data(), _CS_s1.data() + _CS_s1.size(),
                          _CS_s2.data(), _CS_s2.data() + _CS_s2.size());
}

// --------------------------------------------------------------------------

_CXXRT_NAMESPACE_END

#if defined(_CXXRT_LOCALE_IMPL) && !defined(__SRCCXX_COLLATE_CC__)
#  include <srccxx/collate.cc>
#endif

#endif /* _CXXRT_COLLATE_H__ */
