// -*-C++-*- cxxrt/count.h
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 2000 Dietmar Kuehl, Phaidros Software AG > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Phaidros Software AG make > 
// <!! no representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar_kuehl@yahoo.com 
// Title:  Optimized implementation of the 'count()' algorithm
// Version: $Id: count.h,v 1.1 2000/08/08 00:02:35 kuehl Exp $ 

// -------------------------------------------------------------------------- 

#if !defined(_CXXRT_COUNT_H)
#define _CXXRT_COUNT_H 1

#include <cxxrt/iterhelp.h>
#include <cxxrt/iterator.h>
#include <algorithm>

namespace _Cxxrt_algorithm
{

  // ------------------------------------------------------------------------
  // The general and simple case:

  template <class _CS_InIt, class _CS_Pred>
  inline typename _CXXRT_STD iterator_traits<_CS_InIt>::difference_type
  _Cxxrt_count_if(_CS_InIt _CS_beg, _CS_InIt _CS_end, _CS_Pred& _CS_pred, _Cxxrt_bool<true> const&)
  {
    typedef typename _CXXRT_STD iterator_traits<_CS_InIt>::difference_type _CS_Difference;
    _CS_Difference _CS_rc = _CS_Difference();
    
    _CS_beg = _CXXRT_STD find_if(_CS_beg, _CS_end, _CS_pred);
    while (_CS_beg != _CS_end)
      {
	++_CS_rc;
	_CS_beg = _CXXRT_STD find_if(++_CS_beg, _CS_end, _CS_pred);
      }
    
    return _CS_rc;
  }

  // ------------------------------------------------------------------------
  // Hoist test for segments out of the inner loop:

  template <class _CS_InIt, class _CS_Pred>
  inline typename _CXXRT_STD iterator_traits<_CS_InIt>::difference_type
  _Cxxrt_count_if(_CS_InIt _CS_beg, _CS_InIt _CS_end, _CS_Pred& _CS_pred, _Cxxrt_bool<false> const&)
  {
    typedef typename _CXXRT_STD iterator_traits<_CS_InIt>::difference_type _CS_Difference;
    _CS_Difference _CS_rc = _CS_Difference();
    _Cxxrt_iterhelper<_CS_InIt> _CS_helper(_CS_beg, _CS_end);
    
    for (; _CS_helper; ++_CS_helper)
      _CS_rc += _Cxxrt_count_if(_CS_helper.begin(), _CS_helper.end(), _CS_pred,
				_Cxxrt_bool<true>());
    
    return _CS_rc;
  }

  // ------------------------------------------------------------------------
  // Some delegation functions which forward the request to the actual
  // counting function:

  template <class _CS_InIt, class _CS_Pred>
  inline typename _CXXRT_STD iterator_traits<_CS_InIt>::difference_type
  count_if(_CS_InIt _CS_beg, _CS_InIt _CS_end, _CS_Pred _CS_pred)
  {
    return _Cxxrt_count_if(_CS_beg, _CS_end, _CS_pred, _Cxxrt_bool<false>());
  }

  template <class _CS_InIt, class _CS_T>
  inline typename _CXXRT_STD iterator_traits<_CS_InIt>::difference_type
  count(_CS_InIt _CS_beg, _CS_InIt _CS_end, _CS_T const& _CS_val)
  {
    _CXXRT_STD equal_to<_CS_T> eq();
    return ::_Cxxrt_algorithm::count_if(_CS_beg, _CS_end,
					_CXXRT_STD bind1st(_CXXRT_STD equal_to<_CS_T>(), _CS_val));
  }

// --------------------------------------------------------------------------

}

#endif /* _CXXRT_COUNT_H */
