// <!!-*-C++-*- file: _CS_map.h --->
// <!!-------------------------------------------------------------------------->
// <!! Copyright (C) 1997 Dietmar Kuehl >
// <!!   Universitaet Konstanz, Germany, Lehrstuhl fuer praktische Informatik I >
// <!!>
// <!! Permission to use, copy, modify, distribute and sell this >
// <!! software for any purpose is hereby granted without fee, provided >
// <!! that the above copyright notice appears in all copies and that >
// <!! both that copyright notice and this permission notice appear in >
// <!! supporting documentation. The Universitaet Konstanz and the >
// <!! Lehrstuhl fuer praktische Informatik I make no representations >
// <!! about the suitability of this software for any purpose. It is >
// <!! provided "as is" without express or implied warranty. >
// <!!-------------------------------------------------------------------------->

// Author: Dietmar Kuehl dietmar.kuehl@uni-konstanz.de www.informatik.uni-konstanz.de/~kuehl
// Title:  Until the IOStream stuff is implemented, I can't use the standard classes...

// -----------------------------------------------------------------------------

#if !defined(__DKMAP_H__)
#define __DKMAP_H__

// -------------------------------------------------------------------------

_CXXRT_NAMESPACE_BEGIN

// -----------------------------------------------------------------------------

template <class Key, class Value>
class _CS_map
{
public:
  struct item // modeled after 'std::pair'
  {
    Key const first;
    Value     second;
    item*     next;

    ~item() {}
    item(Key const &f, Value const &s = Value()):
      first(f),
      second(s),
      next(0)
    {
    }
  private:
    item(item const &);
    void operator= (item const &);
  };

  class iterator
  {
    friend class _CS_map;
  public:
    iterator(): it(0) {}
    iterator(iterator const &i): it(i.it) {}
    explicit iterator(item *i): it(i) {}

    iterator &operator= (iterator const &i) { it = i.it; return *this; }
    ~iterator() {}

    item& operator* () const { return *it; }
    item* operator-> () { return it; }
    item* operator-> () const { return it; }
    iterator &operator++ () { it = it->next; return *this; }

    bool operator== (iterator const &i) { return it == i.it; }
    bool operator!= (iterator const &i) { return it != i.it; }

#if !defined(_MSC_VER)
  private:
#endif
    item *it;
  };
  typedef iterator const_iterator;

  _CS_map():
    _begin(0)
  {
  }
  ~_CS_map()
  {
		clear();
  }

  iterator begin() { return iterator(_begin); }
  iterator end() { return iterator(); }
  const_iterator begin() const { return iterator(_begin); }
  const_iterator end() const { return const_iterator(); }

  iterator find(Key const &key)
  {
    for (typename _CS_map::iterator it(_begin); it != typename _CS_map::iterator(); ++it)
      if ((*it).first == key)
        return it;
    return typename _CS_map::iterator();
  }
  const_iterator find(Key const &key) const
  {
    for (const_iterator it(_begin); it != iterator(); ++it)
      if ((*it).first == key)
        return const_iterator(it);
    return const_iterator();
  }

  Value &operator[] (Key const &key)
  {
    iterator it(find(key));
    if (it == iterator())
      {
        item *i = new item(key);
        i->next = _begin;
        _begin = i; 
        it = iterator(i);
      }
    return (*it).second;
  }

	void clear()
	{
    for (iterator it(_begin); it != iterator(); )
      {
        iterator tmp = it;
        ++it;
        delete tmp.it;
      }

		_begin = 0;
	}
private:
  _CS_map(_CS_map const &);
  void operator= (_CS_map const &);

  item *_begin;
};

// -----------------------------------------------------------------------------

_CXXRT_NAMESPACE_END

#endif /* __DKMAP_H__ */
