// -*-C++-*- cxxrt/for_each.h
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 2000 Dietmar Kuehl, Phaidros Software AG > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Phaidros Software AG make > 
// <!! no representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar_kuehl@yahoo.com 
// Title:  Optimized implementation of the 'for_each()' algorithm
// Version: $Id: for_each.h,v 1.1 2000/08/14 01:10:09 kuehl Exp $ 

// -------------------------------------------------------------------------- 

#if !defined(_CXXRT_FOR_EACH_H)
#define _CXXRT_FOR_EACH_H 1

#include <cxxrt/iterhelp.h>
#include <cxxrt/iterator.h>
#include <algorithm>

namespace _Cxxrt_algorithm
{

  // ------------------------------------------------------------------------
  // The general and simple case:

  template <class _CS_InIt, class _CS_Func>
  inline void
  _Cxxrt_for_each(_CS_InIt _CS_beg, _CS_InIt _CS_end, _CS_Func& _CS_func, _Cxxrt_bool<true> const&)
  {
    for (; _CS_beg != _CS_end; ++_CS_beg)
      _CS_func(*_CS_beg);
  }

  // ------------------------------------------------------------------------
  // Hoist test for segments out of the inner loop:

  template <class _CS_InIt, class _CS_Func>
  inline void
  _Cxxrt_for_each(_CS_InIt _CS_beg, _CS_InIt _CS_end, _CS_Func& _CS_func, _Cxxrt_bool<false> const&)
  {
    _Cxxrt_iterhelper<_CS_InIt> _CS_helper(_CS_beg, _CS_end);
    
    for (; _CS_helper; ++_CS_helper)
      _Cxxrt_for_each(_CS_helper.begin(), _CS_helper.end(), _CS_func, _Cxxrt_bool<true>());
  }

  // ------------------------------------------------------------------------
  // Some delegation functions which forward the request to the actual
  // counting function:

  template <class _CS_InIt, class _CS_Func>
  inline _CS_Func
  for_each(_CS_InIt _CS_beg, _CS_InIt _CS_end, _CS_Func _CS_func)
  {
    _Cxxrt_for_each(_CS_beg, _CS_end, _CS_func, _Cxxrt_bool<false>());
    return _CS_func;
  }

// --------------------------------------------------------------------------

}

#endif /* _CXXRT_FOR_EACH_H */
