// -*-C++-*- cxxrt/messages.h
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 1998 Dietmar Kuehl, Claas Solutions GmbH > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Claas Solutions make no > 
// <!! representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar.kuehl@claas-solutions.de 
// Title:  Declarations for the messages facet

// -------------------------------------------------------------------------- 

#if !defined(_CXXRT_MESSAGES_H__)
#define _CXXRT_MESSAGES_H__ 1

#if !defined(_CXXRT_LOCALE_H__)
#  include <cxxrt/locale.h>
#endif 
#if !defined(_CXXRT_BSTR_FWD_H__)
#  include <cxxrt/bstr_fwd.h>
#endif

// -------------------------------------------------------------------------

_CXXRT_NAMESPACE_BEGIN

// --------------------------------------------------------------------------

class messages_base
{
public:
	typedef int catalog;
};

// --------------------------------------------------------------------------

template <class _CS_cT>
class messages: public locale::facet, public messages_base
{
public:
	typedef _CS_cT               char_type;
	typedef basic_string<_CS_cT> string_type;

	static locale::id id;

	_CXXRT_EXPLICIT messages(size_t = 0);

	catalog     open(basic_string<char> const&, locale const&) const;
	string_type get(catalog, int, int, string_type const&) const;
	void        close(catalog) const;

protected:
	virtual ~messages();
	
	virtual catalog     do_open(basic_string<char> const&, locale const&) const;
	virtual string_type do_get(catalog, int, int, string_type const&) const;
	virtual void        do_close(catalog) const;
};

// --------------------------------------------------------------------------

template <class _CS_cT>
class messages_byname: public messages<_CS_cT>
{
public:
	typedef _CS_cT               char_type;
	typedef basic_string<_CS_cT> string_type;

	_CXXRT_EXPLICIT messages_byname(char const*, size_t = 0);

protected:
	virtual ~messages_byname();
	
	virtual messages_base::catalog do_open(basic_string<char> const&, locale const&) const;
	virtual string_type do_get(messages_base::catalog, int, int, string_type const&) const;
	virtual void        do_close(messages_base::catalog) const;
};

// --------------------------------------------------------------------------

template <>
inline bool has_facet<messages<char> >(locale const&) 
{
  return true;
}

template <>
inline messages<char> const& use_facet<messages<char> >(locale const& _CS_l)
{
  return _CS_use_facet<messages<char> >(_CS_l);
}

template <>
inline bool has_facet<messages<wchar_t> >(locale const&) 
{
  return true;
}
template <>
inline messages<wchar_t> const& use_facet<messages<wchar_t> >(locale const& _CS_l)
{
  return _CS_use_facet<messages<wchar_t> >(_CS_l);
}

// --------------------------------------------------------------------------

_CXXRT_NAMESPACE_END

#if defined(_CXXRT_LOCALE_IMPL) && !defined(__SRCCXX_MESSAGES_CC__)
#  include <srccxx/messages.cc>
#endif

#endif /* _CXXRT_MESSAGES_H__ */
