// -*-C++-*- cxxrt/moneypunct.h
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 1998 Dietmar Kuehl, Claas Solutions GmbH > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Claas Solutions make no > 
// <!! representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar.kuehl@claas-solutions.de 
// Title:  Declarations for the moneypunct facet

// -------------------------------------------------------------------------- 

#if !defined(_CXXRT_MONEYPUNCT_H__)
#define _CXXRT_MONEYPUNCT_H__ 1

#if !defined(_CXXRTCF_H__)
#  include <cxxrtcf.h>
#endif
#if !defined(_CXXRT_LOCALE_H__)
#  include <cxxrt/locale.h>
#endif 
#if !defined(_CXXRT_STRING_H__)
#  include <cxxrt/string.h>
#endif 

// -------------------------------------------------------------------------

_CXXRT_NAMESPACE_BEGIN

// --------------------------------------------------------------------------

class money_base
{
public:
  enum part
  {
    none,
    space,
    symbol,
    sign,
    value
  };
  struct pattern
  {
    char field[4];
  };
};

template <class _CS_cT, bool _CS_Intl>
class moneypunct: public locale::facet, public money_base
{
public:
  typedef _CS_cT               char_type;
  typedef basic_string<_CS_cT> string_type;

  static locale::id id;
#if _CXXRT_HAS_CONST_EXPRESSION_MEMBER == 1
  static bool const intl = _CS_Intl;
#else
  enum { intl = _CS_Intl };
#endif

  _CXXRT_EXPLICIT moneypunct(size_t = 0);

  char_type          decimal_point() const;
  char_type          thousands_sep() const;
  basic_string<char> grouping() const;
  string_type        curr_symbol() const;
  string_type        positive_sign() const;
  string_type        negative_sign() const;
  int                frac_digits() const;
  pattern            pos_format() const;
  pattern            neg_format() const;
  
protected:
  ~moneypunct();

  virtual char_type          do_decimal_point() const;
  virtual char_type          do_thousands_sep() const;
  virtual basic_string<char> do_grouping() const;
  virtual string_type        do_curr_symbol() const;
  virtual string_type        do_positive_sign() const;
  virtual string_type        do_negative_sign() const;
  virtual int                do_frac_digits() const;
  virtual pattern            do_pos_format() const;
  virtual pattern            do_neg_format() const;
};

// --------------------------------------------------------------------------

template <class _CS_cT, bool _CS_Intl>
class moneypunct_byname: public moneypunct<_CS_cT, _CS_Intl>
{
public:
  typedef money_base::pattern        pattern;
  typedef _CS_cT               char_type;
  typedef basic_string<_CS_cT> string_type;

  _CXXRT_EXPLICIT moneypunct_byname(char const*, size_t = 0);

protected:
  ~moneypunct_byname();

  virtual char_type           do_decimal_point() const;
  virtual char_type           do_thousands_sep() const;
  virtual basic_string<char>  do_grouping() const;
  virtual string_type         do_curr_symbol() const;
  virtual string_type         do_positive_sign() const;
  virtual string_type         do_negative_sign() const;
  virtual int                 do_frac_digits() const;
  virtual money_base::pattern do_pos_format() const;
  virtual money_base::pattern do_neg_format() const;
};

// --------------------------------------------------------------------------

template <>
inline bool has_facet<moneypunct<char, false> >(locale const&) 
{
  return true;
}

template <>
inline moneypunct<char, false> const& use_facet<moneypunct<char, false> >(locale const& _CS_l)
{
  return _CS_use_facet<moneypunct<char, false> >(_CS_l);
}

template <>
inline bool has_facet<moneypunct<char, true> >(locale const&) 
{
  return true;
}
template <>
inline moneypunct<char, true> const& use_facet<moneypunct<char, true> >(locale const& _CS_l)
{
  return _CS_use_facet<moneypunct<char, true> >(_CS_l);
}

template <>
inline bool has_facet<moneypunct<wchar_t, false> >(locale const&) 
{
  return true;
}

template <>
inline moneypunct<wchar_t, false> const& use_facet<moneypunct<wchar_t, false> >(locale const& _CS_l)
{
  return _CS_use_facet<moneypunct<wchar_t, false> >(_CS_l);
}

template <>
inline bool has_facet<moneypunct<wchar_t, true> >(locale const&) 
{
  return true;
}
template <>
inline moneypunct<wchar_t, true> const& use_facet<moneypunct<wchar_t, true> >(locale const& _CS_l)
{
  return _CS_use_facet<moneypunct<wchar_t, true> >(_CS_l);
}

// --------------------------------------------------------------------------

_CXXRT_NAMESPACE_END

#if defined(_CXXRT_LOCALE_IMPL) && !defined(__SRCCXX_MONEYPUNCT_CC__)
#  include <srccxx/moneypunct.cc>
#endif

#endif /* _CXXRT_MONEYPUNCT_H__ */
