// -*-C++-*- srccxx/fstream.cc
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 1997 Dietmar Kuehl, Claas Solutions GmbH > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Claas Solutions make no > 
// <!! representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar.kuehl@claas-solutions.de 
// Title:  Implementation of the file stream classes

// -------------------------------------------------------------------------- 

#if !defined(__SRCCXX_FSTREAM_CC__)
#define __SRCCXX_FSTREAM_CC__ 1

#if !defined(_CXXRT_FSTREAM_H__)
#  include <cxxrt/fstream.h>
#endif

// #include <unistd.h>

#if _CXXRT_IO_TYPEDEFS == 0
  typedef _CXXRT_STREAMSIZE streamsize;
#endif

// -------------------------------------------------------------------------

_CXXRT_NAMESPACE_BEGIN

// --------------------------------------------------------------------------

template <class _CS_charT, class _CS_traits>
_CS_basic_fstream_pbase<_CS_charT, _CS_traits>::_CS_basic_fstream_pbase():
  _CS_m_buf()
{
}

template <class _CS_charT, class _CS_traits>
_CS_basic_fstream_pbase<_CS_charT, _CS_traits>::~_CS_basic_fstream_pbase()
{
}

// --------------------------------------------------------------------------

#if defined(_MSC_VER)
template <class _CS_charT, class _CS_traits>
basic_ifstream<_CS_charT, _CS_traits>::basic_ifstream(basic_ifstream const&):
  _CS_basic_fstream_pbase<_CS_charT, _CS_traits>(),
  basic_ios<_CS_charT, _CS_traits>(&_CS_m_buf),
  basic_istream<_CS_charT, _CS_traits>(&_CS_m_buf)
{
  init(&_CS_m_buf);
}

template <class _CS_charT, class _CS_traits>
void
basic_ifstream<_CS_charT, _CS_traits>::operator= (basic_ifstream const&)
{
}
#endif


template <class _CS_charT, class _CS_traits>
basic_ifstream<_CS_charT, _CS_traits>::basic_ifstream():
  basic_ios<_CS_charT, _CS_traits>(&this->_CS_m_buf),
  basic_istream<_CS_charT, _CS_traits>(&this->_CS_m_buf)
{
  this->init(&this->_CS_m_buf);
}

template <class _CS_charT, class _CS_traits>
basic_ifstream<_CS_charT, _CS_traits>::basic_ifstream(char const* _CS_fname,
                                                                  ios_base::openmode _CS_oflags):
  basic_ios<_CS_charT, _CS_traits>(&_CS_m_buf),
  basic_istream<_CS_charT, _CS_traits>(&_CS_m_buf)
{
  this->init(&_CS_m_buf);
  if (_CS_m_buf.open(_CS_fname, _CS_oflags | ios_base::in) == 0)
    this->setstate(ios_base::failbit);
}

template <class _CS_charT, class _CS_traits>
basic_filebuf<_CS_charT, _CS_traits>*
basic_ifstream<_CS_charT, _CS_traits>::rdbuf() const
{
  return &_CS_m_buf;
}

template <class _CS_charT, class _CS_traits>
bool
basic_ifstream<_CS_charT, _CS_traits>::is_open() const
{
  return _CS_m_buf.is_open();
}

template <class _CS_charT, class _CS_traits>
void
basic_ifstream<_CS_charT, _CS_traits>::open(char const* _CS_fname,
                                                        ios_base::openmode _CS_oflags)
{
  _CS_m_buf.open(_CS_fname, _CS_oflags | ios_base::in);
  this->init(&_CS_m_buf);
}

template <class _CS_charT, class _CS_traits>
void
basic_ifstream<_CS_charT, _CS_traits>::close()
{
  _CS_m_buf.close();
}

// --------------------------------------------------------------------------

#if defined(_MSC_VER)
template <class _CS_charT, class _CS_traits>
basic_ofstream<_CS_charT, _CS_traits>::basic_ofstream(basic_ofstream const&):
  basic_ios<_CS_charT, _CS_traits>(&_CS_m_buf),
  basic_ostream<_CS_charT, _CS_traits>(&_CS_m_buf)
{
}

template <class _CS_charT, class _CS_traits>
void
basic_ofstream<_CS_charT, _CS_traits>::operator= (basic_ofstream const&)
{
}
#endif

template <class _CS_charT, class _CS_traits>
basic_ofstream<_CS_charT, _CS_traits>::basic_ofstream():
  basic_ios<_CS_charT, _CS_traits>(&this->_CS_m_buf),
  basic_ostream<_CS_charT, _CS_traits>(&this->_CS_m_buf)
{
  this->init(&this->_CS_m_buf);
}

template <class _CS_charT, class _CS_traits>
basic_ofstream<_CS_charT, _CS_traits>::basic_ofstream(char const* _CS_fname,
                                                                  ios_base::openmode _CS_oflags):
  basic_ios<_CS_charT, _CS_traits>(&this->_CS_m_buf),
  basic_ostream<_CS_charT, _CS_traits>(&this->_CS_m_buf)
{
  this->_CS_m_buf.open(_CS_fname, _CS_oflags | ios_base::out);
  this->init(&this->_CS_m_buf);
}

template <class _CS_charT, class _CS_traits>
basic_filebuf<_CS_charT, _CS_traits>*
basic_ofstream<_CS_charT, _CS_traits>::rdbuf() const
{
  return &_CS_m_buf;
}

template <class _CS_charT, class _CS_traits>
bool
basic_ofstream<_CS_charT, _CS_traits>::is_open() const
{
  return _CS_m_buf.is_open();
}

template <class _CS_charT, class _CS_traits>
void
basic_ofstream<_CS_charT, _CS_traits>::open(char const* _CS_fname,
					    ios_base::openmode _CS_oflags)
{
  _CS_m_buf.open(_CS_fname, _CS_oflags | ios_base::out);
  this->init(&this->_CS_m_buf);
}

template <class _CS_charT, class _CS_traits>
void
basic_ofstream<_CS_charT, _CS_traits>::close()
{
  _CS_m_buf.close();
}

// --------------------------------------------------------------------------

#if defined(_MSC_VER)
template <class _CS_charT, class _CS_traits>
basic_fstream<_CS_charT, _CS_traits>::basic_fstream(basic_fstream const&):
  _CS_basic_fstream_pbase<_CS_charT, _CS_traits>(),
  basic_ios<_CS_charT, _CS_traits>(&_CS_m_buf),
  basic_iostream<_CS_charT, _CS_traits>(&_CS_m_buf)
{
  init(&_CS_m_buf);
}

template <class _CS_charT, class _CS_traits>
void
basic_fstream<_CS_charT, _CS_traits>::operator= (basic_fstream const&)
{
}
#endif

template <class _CS_charT, class _CS_traits>
basic_fstream<_CS_charT, _CS_traits>::basic_fstream():
  _CS_basic_fstream_pbase<_CS_charT, _CS_traits>(),
  basic_ios<_CS_charT, _CS_traits>(&this->_CS_m_buf),
  basic_iostream<_CS_charT, _CS_traits>(&this->_CS_m_buf)
{
  this->init(&this->_CS_m_buf);
}

template <class _CS_charT, class _CS_traits>
basic_fstream<_CS_charT, _CS_traits>::basic_fstream(char const* _CS_fname,
						    ios_base::openmode _CS_oflags):
  basic_ios<_CS_charT, _CS_traits>(&this->_CS_m_buf),
  basic_iostream<_CS_charT, _CS_traits>(&this->_CS_m_buf)
{
  this->_CS_m_buf.open(_CS_fname, _CS_oflags | ios_base::in | ios_base::out);
  this->init(&this->_CS_m_buf);
}

template <class _CS_charT, class _CS_traits>
basic_filebuf<_CS_charT, _CS_traits>*
basic_fstream<_CS_charT, _CS_traits>::rdbuf() const
{
  return &this->_CS_m_buf;
}

template <class _CS_charT, class _CS_traits>
bool
basic_fstream<_CS_charT, _CS_traits>::is_open() const
{
  return _CS_m_buf.is_open();
}

template <class _CS_charT, class _CS_traits>
void
basic_fstream<_CS_charT, _CS_traits>::open(char const* _CS_fname,
                                                         ios_base::openmode _CS_oflags)
{
  _CS_m_buf.open(_CS_fname, _CS_oflags | ios_base::in | ios_base::out);
  init(&_CS_m_buf);
}

template <class _CS_charT, class _CS_traits>
void
basic_fstream<_CS_charT, _CS_traits>::close()
{
  _CS_m_buf.close();
}

// --------------------------------------------------------------------------

_CXXRT_NAMESPACE_END

#endif /* __SRCCXX_FSTREAM_CC__ */
