// -*-C++-*- src/c_lfile.cc
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 1999 Dietmar Kuehl, Claas Solutions GmbH > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Claas Solutions make no > 
// <!! representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar.kuehl@claas-solutions.de 
// Title:  Implementation of a simple file used to locale data
// Version: $Id: c_lfile.cpp,v 1.1 1999/09/28 23:05:27 kuehl Exp $ 

// -------------------------------------------------------------------------- 

#include "cxxrt/c_lfile.h"
#include "cxxrt/ios_base.h"

// -------------------------------------------------------------------------- 
  
_Locale_file::_Locale_file():
  _CS_m_buffer(0),
  _CS_m_current(0),
  _CS_m_end(0),
  _CS_m_file(),
  _CS_m_ok(false)
{
}

_Locale_file::~_Locale_file()
{
  delete[] _CS_m_buffer;
}
	
bool _Locale_file::_CS_open(char const* fname)
{
  if (_CS_m_file._CS_open(fname, _CXXRT_STD_NAME::ios_base::in))
    {
      _CS_m_buffer = new char[_CS_m_file._CS_bufsiz()];
      _CS_m_current = _CS_m_end = _CS_m_buffer;
      _CS_m_ok = true;
    }
  else
    return false;
  return true;
}

bool _Locale_file::_CS_failed() const
{
  return !_CS_m_ok;
}

bool _Locale_file::_CS_is_open() const
{
  return _CS_m_file._CS_is_open();
}

bool _Locale_file::_CS_read(char& c)
{
  if (!_CS_m_ok)
    return false;
  
  if (_CS_m_current == _CS_m_end)
    {
      if (_CS_m_file._CS_is_open())
	{
	  _CXXRT_STD streamsize sz = _CS_m_file._CS_read(_CS_m_buffer, _CS_m_file._CS_bufsiz());
	  
	  _CS_m_end = _CS_m_buffer + sz;
	  _CS_m_current = _CS_m_buffer;
	}

      if (_CS_m_current == _CS_m_end)
	return _CS_m_ok = false;
    }
  c = *_CS_m_current++;
  return true;
}

bool _Locale_file::_CS_read(int& i)
{
  if (!_CS_m_ok)
    return false;

  i = 0;
  char c;
  if (!_CS_read(c) || c < '0' || '9' < c)
    return _CS_m_ok = false;
  else
    do
      i = i * 10 + c - '0';
    while ((_CS_m_ok = _CS_read(c)) && '0' <= c && c <= '9');

  return _CS_m_ok;
}

bool _Locale_file::_CS_read(char* str, int len)
{
  if (!_CS_m_ok)
    return false;

  while (len--)
    if (!_CS_read(*str++))
      return _CS_m_ok = false;
  *str = 0;
  
  char c;
  return _CS_m_ok = _CS_read(c); // skip the space after the string
}

char const* _Locale_file::_CS_read_string()
{
  if (!_CS_m_ok)
    return false;

  int len;
  if (!_CS_read(len))
    return 0;
  
  char* str = new char[len + 1];
  if (!_CS_read(str, len))
    {
      delete[] str;
      return 0;
    }
  
  return str;
}
