// -*-C++-*- c_locale.cc
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 1998 Dietmar Kuehl, Claas Solutions GmbH > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Claas Solutions make no > 
// <!! representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar.kuehl@claas-solutions.de 
// Title:  Implementation of the locale abstraction

// -------------------------------------------------------------------------- 

#include "cxxrt/c_locale.h"
#include <cstring>
#include <cstdlib>
#include "cxxrt/c_lfile.h"

// -------------------------------------------------------------------------- 

void _Locale_openfile(_Locale_file* lfile, char const* loc, char const* fac)
{
  char buffer[8192];

  _CXXRT_STD strcpy(buffer, "/usr/local/lib/cxxrt/");
  _CXXRT_STD strcat(_CXXRT_STD strcat(buffer, loc), "/");
  _CXXRT_STD strcat(buffer, fac);

  lfile->_CS_open(buffer);
}

// -------------------------------------------------------------------------- 

static char const separator = ';';

static char *extract_part(char const *full, char *part, char const *fname, int count)
{
  if (_CXXRT_STD strcmp(full, "") == 0)
    {
      char const *tmp = _CXXRT_STD getenv(fname);
      if (tmp == 0)
        tmp = _CXXRT_STD getenv("LOCALE");
      return tmp == 0? _CXXRT_STD strcpy(part, "C"): _CXXRT_STD strcpy(part, tmp);
    }
  else if (_CXXRT_STD strchr(full, separator) != 0)
    {
      char const *tmp = full;
      for (;count != 0; --count)
        {
          tmp = _CXXRT_STD strchr(tmp, separator);
          if (tmp == 0)
            return 0;
          tmp = tmp + 1;
        }

      char const *end = _CXXRT_STD strchr(tmp, separator);

      if (end != 0)
        {
          _CXXRT_STD strncpy(part, tmp, end - tmp);
          part[end - tmp] = 0;
          return part;
        }
      else
        return _CXXRT_STD strlen(tmp) == 0? 0: _CXXRT_STD strcpy(part, tmp);
    }
  else
    return _CXXRT_STD strcpy(part, full);

  return part;
}

// -------------------------------------------------------------------------- 

char * _Locale_extract_ctype_name(char const *full, char *part)
{
  return extract_part(full, part, "LC_CTYPE", 0);
}

char * _Locale_extract_numeric_name(char const *full, char *part)
{
  return extract_part(full, part, "LC_NUMERIC", 1);
}

char * _Locale_extract_time_name(char const *full, char *part)
{
  return extract_part(full, part, "LC_TIME", 2);
}

char * _Locale_extract_collate_name(char const *full, char *part)
{
  return extract_part(full, part, "LC_COLLATE", 3);
}

char * _Locale_extract_monetary_name(char const *full, char *part)
{
  return extract_part(full, part, "LC_MONETARY", 4);
}

char * _Locale_extract_messages_name(char const *full, char *part)
{
  return extract_part(full, part, "LC_MESSAGES", 5);
}

// -------------------------------------------------------------------------- 

char *_Locale_compose_name(char *buf,
                           const char *ctype, const char *numeric,
                           const char *time, const char *collate,
                           const char *monetary, const char *messages)
{
  char sep[2] = { separator, 0 };

  _CXXRT_STD strcat(_CXXRT_STD strcpy(buf, ctype), sep);
  _CXXRT_STD strcat(_CXXRT_STD strcat(buf, numeric), sep);
  _CXXRT_STD strcat(_CXXRT_STD strcat(buf, time), sep);
  _CXXRT_STD strcat(_CXXRT_STD strcat(buf, collate), sep);
  _CXXRT_STD strcat(_CXXRT_STD strcat(buf, monetary), sep);
  _CXXRT_STD strcat(buf, messages);

  return buf;
}
