// -*-C++-*- coobj.cc
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 1998 Dietmar Kuehl, Claas Solutions GmbH > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Claas Solutions make no > 
// <!! representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar.kuehl@claas-solutions.de 
// Title:  Definition of the standard output stream objects 

// -------------------------------------------------------------------------- 

#include <cxxrt/ctraits.h>
#include <iostream>
#include <cstdio>
#include <cstring>
#include <streambuf>
#include <cassert>
#include <cstdlib>

extern "C" int write(int, char const*, int);

// -------------------------------------------------------------------------- 

#if !defined(_CXXRT_STANDALONE)

_CXXRT_NAMESPACE_BEGIN
  extern streamsize _CS_std_bufsize;
_CXXRT_NAMESPACE_END

namespace {

  class cstdiobuf: public _CXXRT_STD basic_streambuf<char, _CXXRT_STD char_traits<char> >
  {
  public:
    cstdiobuf(::_CXXRT_STD_NAME::FILE *f);

  protected:
    int        sync();
    int_type   overflow(int_type c);

  private:
    cstdiobuf(cstdiobuf const&);
    void operator= (cstdiobuf const&);
    bool flush();

    ::_CXXRT_STD_NAME::FILE* m_file;
    char                     m_buffer[65536];
  };

  inline bool cstdiobuf::flush()
  {
    _CXXRT_STD size_t sz = pptr() - pbase();
    if (sz > 0)
      {
	setp(m_buffer, static_cast<char*>(m_buffer) + _CXXRT_STD _CS_std_bufsize - 1);
	return  _CXXRT_STD fwrite(pbase(), 1, sz, m_file) == sz;
      }

    return true;
  }

  cstdiobuf::cstdiobuf(::_CXXRT_STD_NAME::FILE *f):
    m_file(f)
  {
    setp(m_buffer, m_buffer);
  }

  int cstdiobuf::sync()
  {
    if (!flush())
      return -1;
    return _CXXRT_STD_NAME::fflush(m_file);
  }

  _CXXRT_STD ostream::int_type cstdiobuf::overflow(_CXXRT_STD ostream::int_type c)
  {
    if (!traits_type::eq_int_type(c, traits_type::eof()))
      {
        *pptr() = _CXXRT_STD char_traits<char>::to_char_type(c);
        pbump(1); // ++sz;
      }

    return flush()? traits_type::not_eof(c): traits_type::eof();
  }

// -------------------------------------------------------------------------- 

  class flush_helper
  {
  public:
    flush_helper(_CXXRT_STD basic_streambuf<char>* sbuf): m_sbuf(sbuf) {}
    ~flush_helper() { m_sbuf->pubsync(); }

    operator _CXXRT_STD basic_streambuf<char>* () { return m_sbuf; }

  private:
    flush_helper(flush_helper const&);
    void operator=(flush_helper const&);

    _CXXRT_STD basic_streambuf<char>* m_sbuf;
  };

// -------------------------------------------------------------------------- 

}

_CXXRT_NAMESPACE_BEGIN

// using _CS_swamp::_impure_ptr;
flush_helper cout_helper(new cstdiobuf(stdout));
flush_helper cerr_helper(new cstdiobuf(stderr));
flush_helper clog_helper(new cstdiobuf(stderr));

basic_ostream<char, char_traits<char> > cout(cout_helper, false);
basic_ostream<char, char_traits<char> > cerr(cerr_helper, true);
basic_ostream<char, char_traits<char> > clog(clog_helper, false);

namespace {
  ios_base::Init _CS_flush_streams;
}

_CXXRT_NAMESPACE_END

#else

#include <unistd.h>

namespace
{
  class stdiobuf:
    public _CXXRT_STD streambuf
  {
  public:
    stdiobuf(int fd): m_fd(fd) { setp(m_buffer, m_buffer + s_size - 1); }
  private:
    int_type overflow(int_type c)
    {
      if (!traits_type::eq_int_type(c, traits_type::eof()))
	{
	  *pptr() = traits_type::to_char_type(c);
	  pbump(1);
	}

      if (_Cxxrt_write(m_fd, m_buffer, pptr() - pbase()) != pptr() - pbase())
	return traits_type::eof();
      setp(m_buffer, m_buffer + s_size - 1);
      return traits_type::not_eof(c);
    }

    int sync()
    {
      return traits_type::eq_int_type(traits_type::eof(), overflow(traits_type::eof()));
    }

    int m_fd;
    enum { s_size = 256 };
    char m_buffer[s_size];
  };

  stdiobuf _Cxxrt_coutbuf(1);
  stdiobuf _Cxxrt_cerrbuf(2);
}

void
_Cxxrt_iostream_flush()
{
  _Cxxrt_coutbuf.pubsync();
}

#if 1
_CXXRT_NAMESPACE_BEGIN

ostream cout(&_Cxxrt_coutbuf);
ostream cerr(&_Cxxrt_cerrbuf, true);
ostream clog(&_Cxxrt_cerrbuf);

_CXXRT_NAMESPACE_END
#endif

#endif
