// -*-C++-*- dcomplex.cpp
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 2000 Dietmar Kuehl, Claas Solutions GmbH > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Claas Solutions make no > 
// <!! representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar.kuehl@claas-solutions.de 
// Title:  Instantiation of complex<double>
// Version: $Id: dcomplex.cpp,v 1.3 2000/07/15 07:51:22 kuehl Exp $ 

// -------------------------------------------------------------------------- 

#include <srccxx/complex.cc>
#include <cmath>

_CXXRT_NAMESPACE_BEGIN

// -------------------------------------------------------------------------- 

void _CS_ddivide(double const& _CS_nomreal, double const& _CS_nomimag,
		 double const& _CS_denomreal, double const& _CS_denomimag,
		 double& _CS_real, double& _CS_imag)
{
  _CS_divide(_CS_nomreal, _CS_nomimag, _CS_denomreal, _CS_denomimag, _CS_real, _CS_imag);
}

// -------------------------------------------------------------------------- 

double abs(complex<double> const& _CS_c)
{
  return _CXXRT_STD sqrt(_CXXRT_STD norm(_CS_c));
}

double arg(complex<double> const& _CS_c)
{
  return _CXXRT_STD atan2(_CS_c.imag(), _CS_c.real());
}

double norm(complex<double> const& _CS_c)
{
  return _CS_c.real() * _CS_c.real() + _CS_c.imag() * _CS_c.imag();
}

complex<double> polar(double const& _CS_a, double const& _CS_phi)
{
  return complex<double>(_CS_a * _CXXRT_STD cos(_CS_phi), _CS_a * _CXXRT_STD sin(_CS_phi));
}

// -------------------------------------------------------------------------- 

basic_ostream<char, char_traits<char> >&
operator<< (basic_ostream<char, char_traits<char> >& _CS_os, complex<double> const& _CS_c)
{
  return _CS_print_complex(_CS_os, _CS_c);
}

basic_ostream<wchar_t, char_traits<wchar_t> >&
operator<< (basic_ostream<wchar_t, char_traits<wchar_t> >& _CS_os, complex<double> const& _CS_c)
{
  return _CS_print_complex(_CS_os, _CS_c);
}

basic_istream<char, char_traits<char> >&
operator>> (basic_istream<char, char_traits<char> >& _CS_is, complex<double>& _CS_c)
{
  return _CS_read_complex(_CS_is, _CS_c);
}

basic_istream<wchar_t, char_traits<wchar_t> >&
operator>> (basic_istream<wchar_t, char_traits<wchar_t> >& _CS_is, complex<double>& _CS_c)
{
  return _CS_read_complex(_CS_is, _CS_c);
}

// -------------------------------------------------------------------------- 

#if defined(__EDG__)
template <> double const _CS_complex_consts<double>::_CS_1 = 1.0;
template <> double const _CS_complex_consts<double>::_CS_2 = _CS_1 + _CS_1;
template <> double const _CS_complex_consts<double>::_CS_10 = _CS_2 + _CS_2 + _CS_2 + _CS_2 + _CS_2;
template <> double const _CS_complex_consts<double>::_CS_inv_log10 = _CS_1 / log(_CS_10);
#else
template <> double const _CS_complex_consts<double>::_CS_1 = 1.0;
template <> double const _CS_complex_consts<double>::_CS_2 = 2.0;
template <> double const _CS_complex_consts<double>::_CS_10 = 10.0;
template <> double const _CS_complex_consts<double>::_CS_inv_log10 = 1.0 / log(10.0);
#endif

template <> complex<double> sin(complex<double> const& c) { return _CS_sin(c); }
template <> complex<double> cos(complex<double> const& c) { return _CS_cos(c); }
template <> complex<double> tan(complex<double> const& c) { return _CS_tan(c); }
template <> complex<double> sinh(complex<double> const& c) { return _CS_sinh(c); }
template <> complex<double> cosh(complex<double> const& c) { return _CS_cosh(c); }
template <> complex<double> tanh(complex<double> const& c) { return _CS_tanh(c); }

template <> complex<double> sqrt(complex<double> const& c) { return _CS_sqrt(c); }
template <> complex<double> exp(complex<double> const& c) { return _CS_exp(c); }
template <> complex<double> log(complex<double> const& c) { return _CS_log(c); }
template <> complex<double> log10(complex<double> const& c) { return _CS_log10(c); }
template <> complex<double> pow(double const&f, complex<double> const& c) { return _CS_pow(f, c); }
template <> complex<double> pow(complex<double> const& c, int n) { return _CS_pow(c, n); }
template <> complex<double> pow(complex<double> const& c, double const& f) { return _CS_pow(c, f); }
template <> complex<double> pow(complex<double> const& c, complex<double> const& e) { return _CS_pow(c, e); }

// -------------------------------------------------------------------------- 

_CXXRT_NAMESPACE_END
