// -*-C++-*- locale_r.cc
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 1997 Dietmar Kuehl, Claas Solutions GmbH > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Claas Solutions make no > 
// <!! representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar.kuehl@claas-solutions.de 
// Title:  Implementation of the locale::_CS_intern member functions

// -------------------------------------------------------------------------- 

#include <cxxrt/locale.h>
#include <cxxrt/lcache.h>
#include <cstring>
#include <cassert>
#include <cstddef>

// -------------------------------------------------------------------------- 

static const _CXXRT_STD_NAME::size_t initial_size = 20;

_CXXRT_NAMESPACE_BEGIN

// -------------------------------------------------------------------------- 

locale::_CS_intern::_CS_intern(char const* str):
  _CS_m_refcount(0),
  _CS_m_size(initial_size),
  _CS_m_array(new facet const*[initial_size]),
  _CS_m_name(strcpy(new char[strlen(str)], str))
{
  for (size_t i = 0; i < _CS_m_size; ++i)
    _CS_m_array[i] = 0;
}

locale::_CS_intern::_CS_intern(_CS_intern const& ip, char const* str):
  _CS_m_refcount(0),
  _CS_m_size(ip._CS_m_size),
  _CS_m_array(new facet const*[_CS_m_size]),
  _CS_m_name(strcpy(new char[strlen(str)], str))
{
  for (size_t i = 0; i < _CS_m_size; ++i)
    {
      _CS_m_array[i] = ip._CS_m_array[i];
      if (_CS_m_array[i] != 0)
        _CS_m_array[i]->_CS_reference();
    }
}

locale::_CS_intern::~_CS_intern()
{
  for (size_t i = 0; i < _CS_m_size; ++i)
    {
      if (_CS_m_array[i] != 0 && _CS_m_array[i]->_CS_release())
        delete _CS_m_array[i];
    }
  delete[] _CS_m_array;
  delete[] _CS_m_name;
}

locale::facet const* locale::_CS_intern::_CS_add(id const& i, facet const* f)
{
  if (f == 0)
    return f;

  id::_CS_init(const_cast<id&>(i));

  f = f->_CS_init(*this);

  f->_CS_reference();
  size_t idx = i._CS_identifier;

  if (_CS_m_size <= idx)
    {
      facet const** tmp = _CS_m_array;
      _CS_m_array = new facet const*[idx + 1];
      size_t _CS_count = 0;
      for (; _CS_count < _CS_m_size; ++_CS_count)
        if ((_CS_m_array[_CS_count] = tmp[_CS_count]) != 0)
	  _CS_m_array[_CS_count]->_CS_reference();
      for (; _CS_count < idx; ++_CS_count)
        _CS_m_array[_CS_count] = 0;
      _CS_m_size = idx + 1;

      delete[] tmp;
    }
  else
    {
      if (_CS_m_array[idx] != 0 && _CS_m_array[idx]->_CS_release())
        delete _CS_m_array[idx];
    }
  _CS_m_array[idx] = f;
  return f;
}

void locale::_CS_intern::_CS_init()
{
#if !defined(_CXXRT_USE_FACET_INSTALLS)
  _CS_m_array[_CS_cache<char>::id._CS_identifier] = new _CS_cache<char>;
  _CS_m_array[_CS_cache<char>::id._CS_identifier]->_CS_init(*this);

  _CS_m_array[_CS_cache<wchar_t>::id._CS_identifier] = new _CS_cache<wchar_t>;
  _CS_m_array[_CS_cache<wchar_t>::id._CS_identifier]->_CS_init(*this);
#endif
}

_CXXRT_NAMESPACE_END
