/*
 * Copyright (c) 1997
 * Silicon Graphics Computer Systems, Inc.
 *
 * Permission to use, copy, modify, distribute and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear
 * in supporting documentation.  Silicon Graphics makes no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 */ 

#include <stdexcept>
#include <cstring>
#include <cxxrt/string.h>

#if defined(__STL_USE_EXCEPTIONS) || \
    !(defined(_MIPS_SIM) && defined(_ABIO32) && _MIPS_SIM == _ABIO32)

#include <stl_string_fwd.h>

__STL_BEGIN_NAMESPACE

__Named_exception::__Named_exception(const string& __str)
{
  strncpy(_M_name, __get_c_string(__str), _S_bufsize);
  _M_name[_S_bufsize - 1] = '\0';
}

const char* __Named_exception::what() const __STL_NOTHROW 
{
  return _M_name;
}

logic_error::logic_error(const string& __s) : __Named_exception(__s) {}
runtime_error::runtime_error(const string& __s) : __Named_exception(__s) {}
domain_error::domain_error(const string& __arg) : logic_error(__arg) {}

invalid_argument::invalid_argument(const string& __arg) : logic_error(__arg) {}
length_error::length_error(const string& __arg) : logic_error(__arg) {}
out_of_range::out_of_range(const string& __arg) : logic_error(__arg) {}
range_error::range_error(const string& __arg) : runtime_error(__arg) {}
overflow_error::overflow_error(const string& __arg) : runtime_error(__arg) {}
underflow_error::underflow_error(const string& __arg) : runtime_error(__arg) {}

void __stl_throw_range_error(const char* __msg) 
{
  throw range_error(__msg);
}

void __stl_throw_length_error(const char* __msg)
{
  throw length_error(__msg);
}

__STL_END_NAMESPACE

#endif /* Not o32, and no exceptions */
