// -*-C++-*- stdiobuf.hpp
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 2000 Dietmar Kuehl, Claas Solutions GmbH > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Claas Solutions make no > 
// <!! representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar.kuehl@claas-solutions.de 
// Title:  A starter version of a "stdiobuf", ie. a stream buffer forwarding to a FILE*

// -------------------------------------------------------------------------- 

#if !defined(STDIOBUF_HPP)
#define STDIOBUF_HPP

#include <streambuf>

// -------------------------------------------------------------------------- 

namespace boost {

  class stdiobuf: public std::basic_streambuf<char, std::char_traits<char> >
  {
  public:
    stdiobuf(std::FILE *f);
    
  protected:
    int_type   underflow();
    int_type   overflow(int_type c);
    int        sync();
    
  private:
    stdiobuf(stdiobuf const&);
    void operator= (stdiobuf const&);
    
    std::FILE* m_file;
    char       m_char[8192];
  };
  
  inline
    stdiobuf::cstdiobuf(std::FILE *f):
    m_file(f)
    {
      setg(m_char, m_char + sizeof(m_char), m_char + sizeof(m_char));
    }
  
  inline
    std::char_traits<char>::int_type stdiobuf::underflow()
    {
      if (eback() == egptr())
	return std::char_traits<char>::eof();
      
      std::streamsize sz = std::fread(m_char, 1, sizeof(m_char), stdin);
      setg(m_char, m_char, m_char + sz);
      return sz > 0? std::char_traits<char>::to_int_type(m_char[0]): std::char_traits<char>::eof();
    }


  inline
    std::char_traits<char>::int_type cstdiobuf::overflow(std::ostream::int_type c)
    {
      if (!traits_type::eq_int_type(c, traits_type::eof()))
	{
	  *pptr() = std::char_traits<char>::to_char_type(c);
	  pbump(1);
	}

      return flush()? traits_type::not_eof(c): traits_type::eof();
    }

  int cstdiobuf::sync()
    {
      if (!flush())
	return -1;
      return std::fflush(m_file);
    }
  
// -------------------------------------------------------------------------- 

#endif /* STDIOBUF_HPP */
