// -*-C++-*- wccvt.cpp
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 2000 Dietmar Kuehl, Phaidros Software AG > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Phaidros Software AG make > 
// <!! no representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar_kuehl@yahoo.com 
// Title:  Implementation of missing wide character conversion function
// Version: $Id: wccvt.cpp,v 1.3 2000/08/24 23:41:40 kuehl Exp $ 

// -------------------------------------------------------------------------- 

#include <cerrno>
#include <cstdio>
#include <cwchar>
#include <stdexcept>
#include <string>
#include <cxxrt/codecvt.h>

_CXXRT_NAMESPACE_BEGIN

// -------------------------------------------------------------------------- 

static codecvt<wchar_t, char, mbstate_t> const* _CXXRT_cvt(bool reset = false)
{
  static codecvt<wchar_t, char, mbstate_t> const* rc = 0;
  if (rc == 0 || reset)
    rc = &use_facet<codecvt<wchar_t, char, mbstate_t> >(locale());
  return rc;
}

// -------------------------------------------------------------------------- 

#if !defined(_CXXRT_HAS_BTOWC) || defined(_CXXRT_WCHAR_IMPL)
wint_t btowc(int c)
{
  if (c == EOF)
    return WEOF;

  mbstate_t   state = mbstate_t();
  char const  in = static_cast<unsigned char>(c);
  char const* inext;
  wchar_t     out;
  wchar_t*    onext;
  
  switch (_CXXRT_cvt()->in(state, &in, &in + 1, inext, &out, &out + 1, onext))
    {
    case codecvt_base::ok:
      return out;
    default:
      return WEOF;
    }
}
#endif

// -------------------------------------------------------------------------- 

#if !defined(_CXXRT_HAS_WCTOB) || defined(_CXXRT_WCHAR_IMPL)
int wctob(wint_t c)
{
  if (c == WEOF)
    return EOF;

  mbstate_t      state = mbstate_t();
  wchar_t const  in = static_cast<wchar_t>(c);
  wchar_t const* inext;
  char           out;
  char*          onext;
  
  switch (_CXXRT_cvt()->out(state, &in, &in + 1, inext, &out, &out + 1, onext))
    {
    case codecvt_base::ok:
      return out;
    default:
      return EOF;
    }
}
#endif

// -------------------------------------------------------------------------- 

#if !defined(_CXXRT_HAS_MBSINIT) || defined(_CXXRT_WCHAR_IMPL)
int mbsinit(mbstate_t const* state)
{
  return (state == 0 || *state == mbstate_t())? 1: 0;
}
#endif


// -------------------------------------------------------------------------- 

#if 0 && (!defined(_CXXRT_HAS_MBRLEN) || defined(_CXXRT_WCHAR_IMPL))
size_t mbrlen(char const* _CXXRT_RESTRICT s, size_t n,
	       mbstate_t* _CXXRT_RESTRICT state)
{
  static mbstate_t intern = mbstate_t();
  return _CXXRT_STD mbrtowc(0, s, n, state? state: &intern);
}
#endif

// -------------------------------------------------------------------------- 

#if !defined(_CXXRT_HAS_MBRTOWC) || defined(_CXXRT_WCHAR_IMPL)
size_t mbrtowc(wchar_t* _CXXRT_RESTRICT to,
	       char const* _CXXRT_RESTRICT s, size_t n,
	       mbstate_t* _CXXRT_RESTRICT state)
{
  static mbstate_t intern = mbstate_t();
  if (s == 0)
    return _CXXRT_STD mbrtowc(0, "", 1, state);

  char const* fnext;
  wchar_t*    tnext;
  wchar_t     tmp;

  if (to == 0)
    to = &tmp;

  switch (_CXXRT_cvt()->in(state? *state: intern, s, s + n, fnext, to, to + 1, tnext))
    {
    case codecvt_base::ok:
      return to == wchar_t()? 0: fnext - s;
    case codecvt_base::partial:
      return size_t(-2);
    default:
      errno = EILSEQ;
      return size_t(-1);
    }
}
#endif

// -------------------------------------------------------------------------- 

#if !defined(_CXXRT_HAS_WCTOMBR) || defined(_CXXRT_WCHAR_IMPL)
size_t wctombr(char* _CXXRT_RESTRICT to,
	       wchar_t from, mbstate_t* _CXXRT_RESTRICT state)
{
  static mbstate_t intern = mbstate_t();
  if (to == 0)
    {
      static char* buf = new char[MB_CUR_MAX]; //-dk:TODO
      return _CXXRT_STD wctombr(buf, L'\0', state);
    }

  wchar_t const* fnext;
  char*          tnext;

  switch (_CXXRT_cvt()->out(state? *state: intern, &from, &from + 1, fnext,
			    to, to + MB_CUR_MAX, tnext))
    {
    case codecvt_base::ok:
      return tnext - to;
    default:
      errno = EILSEQ;
      return size_t(-1);
    }
}
#endif

// -------------------------------------------------------------------------- 

_CXXRT_NAMESPACE_END
