// -*-C++-*- src/wioobj.cc
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 1998 Dietmar Kuehl, Claas Solutions GmbH > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Claas Solutions make no > 
// <!! representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar.kuehl@claas-solutions.de 
// Title:  Definition of the standard wide character stream objects 

// -------------------------------------------------------------------------- 

#include <cxxrt/ctraits.h>
#include <iostream>
#include <cstdio>
#include <cstring>
#include <streambuf>

// -------------------------------------------------------------------------- 

#if 1
namespace
{

  class stdiobuf: public _CXXRT_STD basic_streambuf<wchar_t, _CXXRT_STD char_traits<wchar_t> >
  {
  public:
    stdiobuf(::_CXXRT_STD_NAME::FILE *f);
    typedef _CXXRT_STD basic_streambuf<wchar_t, _CXXRT_STD char_traits<wchar_t> >::int_type int_type;

  protected:
    int        sync();
    int_type   overflow(int_type c);
    int_type   underflow();
    _CXXRT_STD streamsize xsputn(char_type const *s, _CXXRT_STD streamsize n);

  private:
    stdiobuf(stdiobuf const&);
    void operator= (stdiobuf const&);

    ::_CXXRT_STD_NAME::FILE *m_file;
    wchar_t     m_char[1];
  };

  stdiobuf::stdiobuf(::_CXXRT_STD_NAME::FILE *f):
    m_file(f)
  {
    setg(0, 0, 0);
    setp(0, 0);
  }

  int stdiobuf::sync()
  {
    return ::_CXXRT_STD_NAME::fflush(m_file);
  }

  _CXXRT_STD basic_ios<wchar_t, _CXXRT_STD char_traits<wchar_t> >::int_type stdiobuf::underflow()
  {
    // using _CS_swamp::_impure_ptr;
    //-dk:TODO this is real slow!
    if ((::_CXXRT_STD_NAME::fread(m_char, 1, 1, stdin)) != 1)
      return _CXXRT_STD char_traits<wchar_t>::eof();
    setg(m_char, m_char, m_char + 1);
    return _CXXRT_STD char_traits<char>::to_int_type(m_char[0]);
  }

  stdiobuf::int_type stdiobuf::overflow(stdiobuf::int_type c)
  {
    return ::_CXXRT_STD_NAME::fputc(c, m_file);
  }

  _CXXRT_STD streamsize stdiobuf::xsputn(wchar_t const *str, _CXXRT_STD streamsize s)
  {
    return ::_CXXRT_STD_NAME::fwrite(str, 1, s, m_file);
  }

// -------------------------------------------------------------------------- 

}

_CXXRT_NAMESPACE_BEGIN

// using _CS_swamp::_impure_ptr;
basic_istream<wchar_t, char_traits<wchar_t> > wcin (new stdiobuf(stdin), &wcout);
basic_ostream<wchar_t, char_traits<wchar_t> > wcout(new stdiobuf(stdout), false);
basic_ostream<wchar_t, char_traits<wchar_t> > wcerr(new stdiobuf(stderr), true);
basic_ostream<wchar_t, char_traits<wchar_t> > wclog(new stdiobuf(stderr), false);


_CXXRT_NAMESPACE_END
#endif
