/* The following code example is taken from the book
 * "The C++ Standard Library - A Tutorial and Reference"
 * by Nicolai M. Josuttis, Addison-Wesley, 1999
 *
 * (C) Copyright Nicolai M. Josuttis 1999.
 * Permission to copy, use, modify, sell and distribute this software
 * is granted provided this copyright notice appears in all copies.
 * This software is provided "as is" without express or implied
 * warranty, and with no claim as to its suitability for any purpose.
 */
#include <iostream>
#include <streambuf>
#include <cstdio>
#if defined(_CXXRT_STD_NAME) 
#  define STD_NAME _CXXRT_STD_NAME 
#else 
#  define STD_NAME std 
#endif

extern "C" {
    int write (int fd, const char* buf, int num);
}

class fdoutbuf : public STD_NAME::streambuf {
  protected:
    int fd;    // file descriptor
  public:
    // constructor
    fdoutbuf (int _fd) : fd(_fd) {
    }
  protected:
    // write one character
    virtual int_type overflow (int_type c) {
        if (c != EOF) {
            char z = c;
            if (write (fd, &z, 1) != 1) {
                return EOF;
            }
        }
        return c;
    }
    // write multiple characters
    virtual
    STD_NAME::streamsize xsputn (const char* s,
                            STD_NAME::streamsize num) {
        return write(fd,s,num);
    }
};

class fdostream : public STD_NAME::ostream {
  protected:
    fdoutbuf buf;
  public:
    fdostream (int fd) : STD_NAME::ostream(&buf), buf(fd) {
    }
};
