// -*-C++-*- testalloc.hpp
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 2000 Dietmar Kuehl, Claas Solutions GmbH > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Claas Solutions make no > 
// <!! representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar.kuehl@claas-solutions.de 
// Title:  A test allocator
// Version: $Id: testalloc.hpp,v 1.2 2000/07/15 07:51:33 kuehl Exp $ 

// -------------------------------------------------------------------------- 

#if !defined(TESTALLOC_HPP)
#define TESTALLOC_HPP 1

#include <iostream>
#include <new>

#if defined(_ESTD_STD)
#  define STD _ESTD_STD
using STD size_t;
using STD ptrdiff_t;
#elif defined(_CXXRT_STD)
#  define STD _CXXRT_STD
using STD size_t;
using STD ptrdiff_t;
#else
#  define STD ::std::
using namespace std;
#endif

// -------------------------------------------------------------------------- 

namespace tst {

  bool set(bool& val) { return val = true; }

  bool used = false;
  int  allocs = 0;
  int  deallocs = 0;
  int  constructed = 0;
  int  destructed = 0;

  template <class T> class allocator;

  template <>
    class allocator<void>
    {
    public:
      typedef void*       pointer;
      typedef const void* const_pointer;
      typedef void        value_type;
      template <class U>
      struct rebind
      {
	typedef allocator<U> other;
      };
    };
  
  template <class T>
    class allocator
    {
    public:
      static void report()
	{
	  if (used)
	    {
	      if (allocs == deallocs && constructed == destructed)
		STD cout << "allocator successfully used\n";
	      else
		{
		  if (allocs < deallocs)
		    STD cout << "more memory released than allocated with allocator (" << allocs << " vs. " << deallocs << ")\n";
		  else if (allocs > deallocs)
		    STD cout << "memory leak when using allocator (" << allocs << " vs. " << deallocs << ")\n";
		  
		  if (constructed < destructed)
		    STD cout << "more objects destructed than constructed with allocator (" << constructed << " vs. " << destructed << ")\n";
		  else if (constructed > destructed)
		    STD cout << "objects not destructed with allocator (" << constructed << " vs. " << destructed << ")\n";
		}
	    }
	  else
	    STD cout << "allocator was never used\n";
	}

      typedef size_t    size_type;
      typedef ptrdiff_t difference_type;
      typedef T*        pointer;
      typedef T const*  const_pointer;
      typedef T&        reference;
      typedef T const&  const_reference;
      typedef T         value_type;

      template <class U>
      struct rebind
      {
	typedef allocator<U> other;
      };

      allocator(bool f = true) throw(): m_flag(f) {}
      allocator(allocator const& a) throw(): m_flag(a.m_flag) {}
      template <class U>
      allocator(allocator<U> const& a) throw(): m_flag(a.m_flag) {}
      ~allocator() throw()                         {}

      pointer address(reference ref) const             { return &ref; }
      const_pointer address(const_reference ref) const { return &ref; }

      pointer allocate(size_type sz, void const* = 0)
	{
	  if (m_flag && set(used))
	    ++allocs;
	  return static_cast<pointer>(operator new(sz));
	}
      void deallocate(pointer ptr, size_type)
	{
	  if (m_flag && set(used))
	    ++deallocs;
	  operator delete(ptr);
	}

      size_type max_size() const throw() { return size_type(-1); }
      
      void construct(pointer ptr, T const& obj)
	{
	  if (m_flag && set(used))
	    ++constructed;
	  new(ptr) T(obj);
	  
	}
      void destroy(pointer ptr)
	{
	  if (m_flag && set(used))
	    ++destructed;
	  ptr->~T();
	}

      bool m_flag;
    };

  // ------------------------------------------------------------------------

  template <class T1, class T2>
    bool
    operator== (allocator<T1> const&, allocator<T2> const&)
  {
    return true;
  }

  template <class T1, class T2>
    bool
    operator!= (allocator<T1> const&, allocator<T2> const&)
  {
    return false;
  }

}

// -------------------------------------------------------------------------- 

#endif /* TESTALLOC_HPP */
