// -*-C++-*- testnumeric.hpp
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 2000 Dietmar Kuehl, Claas Solutions GmbH > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Claas Solutions make no > 
// <!! representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar.kuehl@claas-solutions.de 
// Title:  a type used to test instantiation of the numeric classes
// Version: $Id: testnumeric.hpp,v 1.1 2000/05/06 18:25:05 kuehl Exp $ 

// -------------------------------------------------------------------------- 

#if !defined(TESTNUMERIC_HPP)
#define TESTNUMERIC_HPP 1

namespace tst
{

// --------------------------------------------------------------------------

  class number
  {
  public:
    number(): m_val(0.0) {}
    explicit number(double v): m_val(v) {}
    number(number const& n): m_val(n.m_val) {}
    ~number() {}
    number& operator= (number const& n) { m_val = n.m_val; return *this; }
    number& operator= (double const& n) { m_val = n; return *this; }

    number& operator+= (number const& n) { m_val += n.m_val; return *this; }
    number& operator-= (number const& n) { m_val -= n.m_val; return *this; }
    number& operator*= (number const& n) { m_val *= n.m_val; return *this; }
    number& operator/= (number const& n) { m_val /= n.m_val; return *this; }

    number operator+ () const { return *this; }
    number operator- () const { return number(-m_val); }

    bool operator== (number const& n) const { return m_val == n.m_val; }
    bool operator!= (number const& n) const { return m_val != n.m_val; }
    bool operator<= (number const& n) const { return m_val <= n.m_val; }
    bool operator<  (number const& n) const { return m_val <  n.m_val; }
    bool operator>= (number const& n) const { return m_val >= n.m_val; }
    bool operator>  (number const& n) const { return m_val >  n.m_val; }

    double value() const { return m_val; }
    double& value() { return m_val; }

  private:
    double m_val;
  };

  number operator+ (number n1, number const& n2) { return n1 += n2; }
  number operator- (number n1, number const& n2) { return n1 -= n2; }
  number operator* (number n1, number const& n2) { return n1 *= n2; }
  number operator/ (number n1, number const& n2) { return n1 /= n2; }

  std::ostream& operator<< (std::ostream& os, number const& n) { return os << n.value(); }
  std::istream& operator>> (std::istream& is, number& n) { return is >> n.value(); }

  template <class T>
  bool feq(T const& v1, T const& v2)
    {
      T diff = (v1 - v2) < T(0.0)? -(v1 - v2): (v1 - v2);
      if (v1 < T(0.0000001))
	return diff < T(0.0000001)? true: false;
      return diff < (v1 < T(0.0)? -v1: v1) / T(100000.0)? true: false;
    }

// --------------------------------------------------------------------------

} // namespace tst

#endif /* TESTNUMERIC_HPP */
