// -*-C++-*- tstinc/hexcvt.h
// <!!----------------------------------------------------------------------> 
// <!! Copyright (C) 1999 Dietmar Kuehl, Claas Solutions GmbH > 
// <!!> 
// <!! Permission to use, copy, modify, distribute and sell this > 
// <!! software for any purpose is hereby granted without fee, provided > 
// <!! that the above copyright notice appears in all copies and that > 
// <!! both that copyright notice and this permission notice appear in > 
// <!! supporting documentation. Dietmar Kuehl and Claas Solutions make no > 
// <!! representations about the suitability of this software for any > 
// <!! purpose. It is provided "as is" without express or implied warranty. > 
// <!!----------------------------------------------------------------------> 

// Author: Dietmar Kuehl dietmar.kuehl@claas-solutions.de 
// Title:  A simple test conversion: ASCII <-> code in hex
// Version: $Id: hexcvt.h,v 1.2 1999/09/28 23:05:39 kuehl Exp $ 

// -------------------------------------------------------------------------- 

#if !defined(TSTINC_HEXCVT_H)
#define TSTINC_HEXCVT_H 1

#include <locale>

namespace tst
{

template <typename T>
T const* find(T const* beg, T const* end, T val)
  {
    while (beg != end && *beg != val)
      ++beg;
    return beg;
  }

template <typename T, int sz>
T const* begin(T const (&array)[sz]) { return array; }
template <typename T, int sz>
T const* end(T const (&array)[sz]) { return static_cast<T const*>(array) + sz; }

// --------------------------------------------------------------------------

class hexcvt: public std::codecvt<char, char, std::mbstate_t>
{
protected:
  int do_encoding() throw() { return 2; }

  result do_out(std::mbstate_t& s,
		char const* fb, char const* fe, char const*& fn,
                char* tb, char* te, char*& tn) const
    {
      static char const hex[] = "0123456789abcdef";

      tn = tb;

      if (s.count == 1)
	{
	  if (tn == te)
	    return partial;
	  *tn = hex[s.value % 16];
	  s.count = 0;
	}

      for (fn = fb; fn != fe; ++fn)
	{
	  int c = *fn;
	  if (tn == te)
	    return partial;
	  *tn++ = hex[c / 16];
	  if (tn == te)
	    {
	      s.count = 1;
	      s.value = c;
	      ++fn;
	      return partial;
	    }
	  *tn++ = hex[c % 16];
	}

      return ok;
    }

  result do_in(std::mbstate_t& s,
		char const* fb, char const* fe, char const*& fn,
                char* tb, char* te, char*& tn) const
    {
      static char const hex[] = "0123456789abcdef";
      char const* found;

      tn = tb;
      fn = fb;

      if (s.count != 0)
	{
	  if ((found = find(begin(hex), end(hex) + 16, *fn++)) == end(hex))
	    return error;
	  s.value += found - begin(hex);
	  *tn++ = s.value;
	  s.count = 0;
	}

      while (fn != fe)
	{
	  if (tn == te)
	    return partial;

	  if ((found = find(begin(hex), end(hex) + 16, *fn++)) == end(hex))
	    return error;
	  int c = 16 * (found - begin(hex));

	  if (fn == fe)
	    {
	      s.count = 1;
	      s.value = c;
	      return partial;

	    }
	  if ((found = find(begin(hex), end(hex) + 16, *fn++)) == end(hex))
	    return error;
	  c += found - begin(hex);
	  *tn++ = c;
	}

      return ok;
    }
};

// --------------------------------------------------------------------------

} // namespace tst

#endif /* TSTINC_HEXCVT_H */
